" Vim syntax file
" Language: POV-Ray v3.8 (povr fork) Initialization (.ini) files
" Maintainer: William F. Pokorny
" URLs:
"    http://news.povray.org/povray.binaries.programming/
"    http://news.povray.org/povray.pov4.discussion.general/
" Copyright:
"    Copyright (c) 1991-2023 Persistence of Vision Raytracer Pty. Ltd.
" License:
"    This work is licensed under the Creative Commons Attribution-ShareAlike 4.0
"    International Public License.
"    Visit https://creativecommons.org/licenses/by-sa/4.0/
" Last Change: Sun May  7 19:07:04 -0400 2023

if exists("b:current_syntax")
    finish
endif

" TODO. Why do we see the echo below twice when uncommented?
" echom "Debugging. Reading povrini.vim."

let s:cpo_save = &cpo
set cpo&vim

" POV-Ray ini files and command line options are case agnostic
syn case ignore

" Comments. POV-Ray ini files support ';' and '#' anywhere in line
syn match povriniComment "[#;].*$" contains=povrTodo
syn keyword povriniTodo TODO FIXME NB NOTE contained
syn match povriniCommentErrors "[a-z,A-Z,0-9_\.][#;]\+\ze.*$"

" String literals. Used with file names having spaces (not recommended).
" Note. The povr command line forms require traditional linux/unix double quote escapes.
" Official POV-Ray releases require odd escape sequences. Ref: Github issue 448.
syn region  povriniString        start=+"+ skip=+\\\\\|\\"+ end=+"+ contains=povriniStringSpecial
syn match   povriniStringSpecial "\\x\x\+\|\\u\x\{1,6\}" contained containedin=povriniString
syn match   povriniStringErrors  "\\u\x\{7,\}" contained containedin=povriniString

" Active command line / ini file options
syn keyword povriniKeywords all_console all_file antialias antialias_confidence antialias_depth antialias_gamma
syn keyword povriniKeywords antialias_threshold append_file bits_per_color bits_per_colour bounding bounding_method
syn keyword povriniKeywords bounding_threshold bsp_baseaccesscost bsp_childaccesscost bsp_isectcost bsp_maxdepth
syn keyword povriniKeywords bsp_misschance clock clockless_animation compression continue_trace create_continue_trace_log
syn keyword povriniKeywords create_ini cyclic_animation debug_console debug_file declare display display_gamma
syn keyword povriniKeywords dither dither_method end_column end_row fatal_console fatal_error_command fatal_error_return
syn keyword povriniKeywords fatal_file file_gamma final_clock final_frame frame_step grayscale_output greyscale_output
syn keyword povriniKeywords height high_reproducibility include_header include_ini initial_clock initial_frame
syn keyword povriniKeywords input_file_name jitter jitter_amount library_path max_image_buffer_memory output_alpha
syn keyword povriniKeywords output_file_name output_file_type output_to_file pause_when_done post_frame_command
syn keyword povriniKeywords post_frame_return post_scene_command post_scene_return pre_frame_command pre_frame_return
syn keyword povriniKeywords pre_scene_command pre_scene_return preview_end_size preview_start_size quality radiosity_file_name
syn keyword povriniKeywords radiosity_from_file radiosity_to_file radiosity_vain_pretrace real_time_capture real_time_raytracing
syn keyword povriniKeywords remove_bounds render_block_size render_block_step render_console render_file render_pattern
syn keyword povriniKeywords sampling_method split_unions start_column start_row statistic_console statistic_file
syn keyword povriniKeywords stochastic_seed subset_end_frame subset_start_frame user_abort_command user_abort_return
syn keyword povriniKeywords verbose version warning_console warning_file warning_level width work_threads

" Command line / ini file options removed from povr
syn keyword povriniKeywordErrors buffer_output buffer_size create_histogram draw_vistas field_render histogram_grid_size
syn keyword povriniKeywordErrors histogram_name histogram_type light_buffer odd_field palette radiosity test_abort
syn keyword povriniKeywordErrors test_abort_count video_mode vista_buffer

" Active command line / ini file flags
" ====================================
" Flag form of antialias(a) and antialias_threshold(value)
syn match povriniFlags "[-+]a\(\s\+\|$\)"
syn match povriniFlags "[-+]a[0-9.]\+\(\s\+\|$\)"
" Flag form of antialias_confidence
syn match povriniFlags "[-+]ac0\.[0-9]\+\(\s\+\|$\)"
" Flag form of antialias_gamma
syn match povriniFlags "[-+]ag[0-9.]\+\(\s\+\|$\)"
" Flag form of sampling_method
syn match povriniFlags "[-+]am[1-3]\{1\}\(\s\+\|$\)"

" Flag form of bounding
syn match povriniFlags "[-+]b\(\s\+\|$\)"
" Flag form of bounding_method(1 or 2)
syn match povriniFlags "[-+]bm[1-2]\{1\}\(\s\+\|$\)"

" Flag form of render_block_size
syn match povriniFlags "[-+]bs\([4-9]\{1\}\|[1-9][0-9]\+\)\(\s\+\|$\)"

" Flag form of continue_trace. Only works if +cc created state file in prior partial render 
syn match povriniFlags "[-+]c\(\s\+\|$\)"
" Flag form of create_continue_trace_log. Default is on. 
" Note. Slows render and files can be large. Use -cc, if confidence in render completion high.
syn match povriniFlags "[-+]cc\(\s\+\|$\)"

" Flag form of display. Only works if preview display option(s) compiled into POV-Ray.
syn match povriniFlags "[-+]d\(\s\+\|$\)"

" Flag form of preview_end_size
syn match povriniFlags "[-+]ep[1-9]\{1\}[0-9]*\(\s\+\|$\)"

" Flag form of subset_end_frame
syn match povriniFlags "[-+]ef[0-9]\+[\.]*[0-9]*\(\s\+\|$\)"

" Flag form of output_file_type
syn match povriniFlags "[-+]\(f\|f[b-c,e,h,j,n,p,s-t]\{1\}g\{,1\}\([1-9]\{,1\}\|1[0-6]\{,1\}\)\)\(\s\+\|$\)"

" Flag form of end_column. Pixel count or percentage.
syn match povriniFlags "[-+]\(ec100\|ec[0-9]\+[\.]*[0-9]*\)\(\s\+\|$\)"
" Flag form of end_row. Pixel count or percentage.
syn match povriniFlags "[-+]\(er100\|er[0-9]\+[\.]*[0-9]*\)\(\s\+\|$\)"

" Flag form of Create_Ini=file
syn match povriniFlags "[-+]gi\(\s\+\|$\)"
syn match povriniFlags "[-+]gi\ze\""
syn match povriniFlags "[-+]gi\S\+\(\s\+\|$\)"

" Output stream control
" ---------------------
" Note! Since the introduction of threading it's been the case some output
" is written directly to stdout/cout and stderr/cerr; often from the render threads.
" The status partly code work never completed. However, it is also due re-arranging 
" code with the aim of making the differing parts independent. This bit is work 
" toward callable libraries - where the defined console streams make less sense. 
" ---
" Flag form of debug_console=on/off and debug_file=file
syn match povriniFlags "\([-+]gd\ze\"\)\|\([-+]gd\S\+\(\s\+\|$\)\)\|\([-+]gd\(\s\+\|$\)\)"
syn match povriniFlagErrors "\([-+]gd\s\+\ze\"\)\|\([-+]gd\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of fatal_console=on/off and fatal_file=file
syn match povriniFlags "\([-+]gf\ze\"\)\|\([-+]gf\S\+\(\s\+\|$\)\)\|\([-+]gf\(\s\+\|$\)\)"
syn match povriniFlagErrors "\([-+]gf\s\+\ze\"\)\|\([-+]gf\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of render_console=on/off and render_file=file
syn match povriniFlags "\([-+]gr\ze\"\)\|\([-+]gr\S\+\(\s\+\|$\)\)\|\([-+]gr\(\s\+\|$\)\)"
syn match povriniFlagErrors "\([-+]gr\s\+\ze\"\)\|\([-+]gr\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of statistic_console=on/off and statistic_file=file
syn match povriniFlags "\([-+]gs\ze\"\)\|\([-+]gs\S\+\(\s\+\|$\)\)\|\([-+]gs\(\s\+\|$\)\)"
syn match povriniFlagErrors "\([-+]gs\s\+\ze\"\)\|\([-+]gs\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of warning_console=on/off and warning_file=file
syn match povriniFlags "\([-+]gw\ze\"\)\|\([-+]gw\S\+\(\s\+\|$\)\)\|\([-+]gw\(\s\+\|$\)\)"
syn match povriniFlagErrors "\([-+]gw\s\+\ze\"\)\|\([-+]gw\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of all_console=on/off and all_file=file
syn match povriniFlags "\([-+]ga\ze\"\)\|\([-+]ga\S\+\(\s\+\|$\)\)\|\([-+]ga\(\s\+\|$\)\)"
syn match povriniFlagErrors "\([-+]ga\s\+\ze\"\)\|\([-+]ga\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of append_file boolean for all output streams.
syn match povriniFlags "[-+]gp\(\s\+\|$\)"

" Flag form of height
syn match povriniFlags "[-+]h[1-9]\{1\}[0-9]*\(\s\+\|$\)"

" Flag form of include_header
syn match povriniFlagErrors "[-+]hi\(\s\+\|$\)"
syn match povriniFlags "[-+]hi\ze\""
syn match povriniFlags "[-+]hi\S\+\(\s\+\|$\)"

" Flag form of high_reproducibility
" Note. Works to significant exent, but mostly not as well rendering with a single thread.
syn match povriniFlags "[-+]hr\(\s\+\|$\)"

" Flag form of input_file_name
syn match povriniFlags "\([-+]i\ze\"\)\|\([-+]i\S\+\(\s\+\|$\)\)"
syn match povriniFlagErrors "\([-+]i\s\+\ze\"\)\|\([-+]i\s\+\ze\)\|\([-+]i\ze$\)"

" Flag form of jitter(j) and optional jitter_ammount(value)
syn match povriniFlags "[-+]\(j\|j[0-9]\+[\.]*[0-9]*\)\(\s\+\|$\)"

" Flag form of clock
syn match povriniFlags "[-+]k[0-9]\+[\.]*[0-9]*\(\s\+\|$\)"
" Flag form of initial_frame
" Note! To render subsets of frames use subset frame options
syn match povriniFlags "[-+]kfi[0-9]\+\(\s\+\|$\)"
" Flag form of final_frame
syn match povriniFlags "[-+]kff[0-9]\+\(\s\+\|$\)"
" Flag form of initial_clock
syn match povriniFlags "[-+]ki[0-9]\+[\.]*[0-9]*\(\s\+\|$\)"
" Flag form of final_clock
syn match povriniFlags "[-+]kf[0-9]\+[\.]*[0-9]*\(\s\+\|$\)"

" Flag form of cyclic_animation. Note. In povr this used with normal animation
" renders and the clockless/real time raytracing animations. Default is -kc
syn match povriniFlags "[-+]kc\(\s\+\|$\)"
" Flag form of clockless_animation. Note. Used today only with +rtr.
syn match povriniFlags "[-+]kla\(\s\+\|$\)"

" Flag form of library_path
" Note. Multiples of this flag may be used to include additional directories.
syn match povriniFlags "\([-+]l\ze\"\)\|\([-+]l\S\+\(\s\+\|$\)\)"
syn match povriniFlagErrors "\([-+]l\s\+\ze\"\)\|\([-+]l\s\+\ze\)\|\([-+]l\ze$\)"

" Flag form of bounding_threshold(integer value)
syn match povriniFlags "[-+]mb[0-9]\+\(\s\+\|$\)"

" Flag form of max_image_buffer_memory
" Note. IIRC this setting a concern more for POV-Ray running on windows with file backing.
syn match povriniFlags "[-+]mi[1-9]\{1\}[0-9]*\(\s\+\|$\)"

" Flag form of version
syn match povriniFlags "[-+]mv\([3]\{1\}\.[5-8]\|[4]\{1\}\.[0-1]\)\(\s\+\|$\)"

" Flag form of output_file_name
syn match povriniFlags "\([-+]o\ze\"\)\|\([-+]o\S\+\(\s\+\|$\)\)"
syn match povriniFlagErrors "\([-+]o\s\+\ze\"\)\|\([-+]o\s\+\ze\)\|\([-+]o\ze$\)"

" Flag form of pause_when_done.
syn match povriniFlags "[-+]p\(\s\+\|$\)"

" Flag form of quality
" Note. Levels 10 and 11 are not currently used. So, 9 the highest.
syn match povriniFlags "[-+]q\([0-9]\{1\}\|10\|11\)\(\s\+\|$\)"

" Flag form of antialias_depth
syn match povriniFlags "[-+]r[1-9]\+\(\s\+\|$\)"

" Flag form of render_pattern
syn match povriniFlags "[-+]rp\([0-5]\{1\}\)\(\s\+\|$\)"
" Flag form of render_block_step
syn match povriniFlags "[-+]rs\([1-9]\{1\}[0-9]*\)\(\s\+\|$\)"

" Flag form of Radiosity_File
syn match povriniFlags "\([-+]rf\ze\"\)\|\([-+]rf\S\+\(\s\+\|$\)\)"
syn match povriniFlagErrors "\([-+]rf\s\+\ze\"\)\|\([-+]rf\s\+\ze\)\|\([-+]rf\ze$\)"
" Flag form of Radiosity_From_File
syn match povriniFlags "[-+]rfi\(\s\+\|$\)"
" Flag form of Radiosity_To_File
syn match povriniFlags "[-+]rfo\(\s\+\|$\)"
" Flag form of radiosity_vain_pretrace
syn match povriniFlags "[-+]rvp\(\s\+\|$\)"

" --- real time raytracing / image capture 
" Flag form of real_time_capture. Note. This is a povr only option which today
" writes to numbered, color pfm files. The image file creation also bypasses the
" usual povmsg data to image buffer date passing which often backs up causing huge
" storage bubbles.

" The color pfm output is the only true 32bit/channel POV-Ray image output. The
" hdr and exr (currently only half float) are good for dynamic range only. There
" is currently no POV-Ray (povr) pfm input capability. Outside utilities must convert
" to other more mainstream formats for ffmpeg or POV-Ray use as input.

" Note. Single image renders to pfm output can be accomplished by using:
" +rtr +rtc -kla. The key being -kla. Only the first camera of one or more
" cameras will be rendered and written to a pfm file.
syn match povriniFlags "[-+]rtc\(\s\+\|$\)"
" Flag form of real_time_raytracing. This works only with the clockless, camera 
" list kind of animations. In other words +kla and +rtr must be used together. 
syn match povriniFlags "[-+]rtr\(\s\+\|$\)"
" ---

" Flag form of start_column. Pixel count or percentage.
syn match povriniFlags "[-+]\(sc0\|sc[0-9]\+[\.]*[0-9]*\)\(\s\+\|$\)"
" Flag form of start_row. Pixel count or percentage.
syn match povriniFlags "[-+]\(sr0\|sr[0-9]\+[\.]*[0-9]*\)\(\s\+\|$\)"

" Flag form of subset_start_frame
syn match povriniFlags "[-+]sf[0-9]\+[\.]*[0-9]*\(\s\+\|$\)"

" Flag form of preview_start_size
syn match povriniFlags "[-+]sp[1-9]\{1\}[0-9]*\(\s\+\|$\)"

" Flag form of frame_step
syn match povriniFlags "[-+]stp[0-9]\+\(\s\+\|$\)"

" Flag form of split_unions (default is off)
" Note. Manual bounding can set photon shooting target to entire union.
syn match povriniFlags "[-+]su\(\s\+\|$\)"
" Flag form of stochastic_seed(value)
syn match povriniFlags "[-+]ss[0-9]\+\(\s\+\|$\)"

" Flag form of dither and dither_mode
syn match povriniFlags "\([-+]th\(\s\+\|$\)\)\|\([-+]th\(b2\|b3\|b4\|bn\|bnx\|at\|bk\|d1\|d2\|fs\|jn\|s2\|s3\|st\)\(\s\+\|$\)\)"

" Flag form of remove_bounds (default is off)
" Note. Must also have background / env transparency in scene - otherwise pointless
syn match povriniFlags "[-+]ua\(\s\+\|$\)"

" Flag form of remove_bounds (default is on)
" Note. There are exceptions where bounding is not removed
" Plus. There are cases where users should NOT remove bounds due poor or no auto-bounding
syn match povriniFlags "[-+]ur\(\s\+\|$\)"

" Flag form of verbose. Default is on. Most noticeable effect is text progress messages.
syn match povriniFlags "[-+]v\(\s\+\|$\)"

" Flag form of width
syn match povriniFlags "[-+]w\([1-9]\{1\}[0-9]*\)\(\s\+\|$\)"
" Flag form of warning_level. Note. Support for this feature has eroded over time.
syn match povriniFlags "[-+]wl[0-9]\{1,2\}\(\s\+\|$\)"
" Flag form of work_threads
syn match povriniFlags "[-+]wt[0-9]\+\(\s\+\|$\)"

" Active ini file options without flag equivalents.
" ---
" (You can add long ini forms to the command line)
" ====================================
" compression=<target_ratio> for jpeg output
" declare=Var00=0.123
" display_gamma=(n.n|bt709|bt2020|srgb)
" file_gamma=(n.n|bt709|bt2020|srgb)
" include_ini=<filename> // Note. This explicit form, safer than guessed raw filename.
" --- Shell command options
" pre_scene_command=s
" pre_frame_command=s
" post_scene_command=s
" post_frame_command=s
" user_abort_command=s
" fatal_abort_command=s
" -
" where the string substitutions are:
" %o %s %n %k %h %w %%
" --- Shell command return actions
" pre_scene_return=s
" pre_frame_return=s
" post_scene_return=s
" post_frame_return=s
" user_abort_return=s
" fatal_abort_return=s
" -
" where the string substitutions are:
" i s a q u f 
" ---
" (The animation command optios only apply to full animation and not RTR.)
" ---

" Command line / ini file flags removed from povr and deprecated in the official v3.8
" ====================================
" Flag form of video_mode or pallete.
" Note. Video color mode and pallete no longer needed with modern displays.
" POV-Ray allows bogus settings in v3.7 on, povr doesn't. 
syn match povriniFlagErrors "[-+]d\S\+\(\s\+\|$\)"

" Flag form of histogram_file
syn match povriniFlagErrors "\([-+]hn[^\"]\ze\)\|\([-+]hn\s*\ze\"\)\|\([-+]hn\)\(\s\+\|$\)"
" Flag form of histogram_grid_size
syn match povriniFlagErrors "\([-+]hs\(\s\+\|$\)\)\|\([-+]hs\d\+\.\d\+\(\s\+\|$\)\)"
" Flag form of histogram_type
syn match povriniFlagErrors "[-+]ht\(\s\+\|$\)\|[-+]ht\(c\|n\|p\|s\|t\|x\)\(\s\+\|$\)"

" Flag form of radiosity. This long triggered with SDL radiosity block instead.
syn match povriniFlagErrors "[-+]qr\(\s\+\|$\)"

" Flag form of draw_vistas
syn match povriniFlagErrors "[-+]ud\(\s\+\|$\)"
" Flag form of light_buffer
syn match povriniFlagErrors "[-+]ul\(\s\+\|$\)"
" Flag form of vista_buffer
syn match povriniFlagErrors "[-+]uv\(\s\+\|$\)"

" Flag form of field_render=bool
syn match povriniFlagErrors "[-+]uf\(\s\+\|$\)"
" Flag form of odd_field=bool
syn match povriniFlagErrors "[-+]uo\(\s\+\|$\)"

" Flag form of test_abort=on|off and test_abort_count <positive integer>
" Note. The regex can precent leadng first 0, but current regex allows it
syn match povriniFlagErrors "[-+]\(x\|x[0-9]\{1\}[0-9]*\)\(\s\+\|$\)"

" POV-Ray (povr form) Unix / linux command line flags
" ===================================================
" Note! These are NOT ini file options and only work from the command line
" ===================================================
" Preview display options. This overrides +-d.
syn match povrUnixFlags "[-]\{2\}preview\s\+x11\(\s\+\|$\)"
syn match povrUnixFlags "[-]\{2\}preview\s\+sdl\(\s\+\|$\)"
syn match povrUnixFlags "[-]\{2\}preview\s\+text\(\s\+\|$\)"
syn match povrUnixFlags "[-]\{1\}y\s\+x11\(\s\+\|$\)"
syn match povrUnixFlags "[-]\{1\}y\s\+sdl\(\s\+\|$\)"
syn match povrUnixFlags "[-]\{1\}y\s\+text\(\s\+\|$\)"

" General options
syn match povrUnixFlags "[-]\{1,2\}authors\(\s\+\|$\)"
syn match povrUnixFlags "[-]\{2\}generation\(\s\+\|$\)"
syn match povrUnixFlags "[-]\{1,2\}help\(\s\+\|$\)"
syn match povrUnixFlags "[-]h\(\s\+\|$\)"
syn match povrUnixFlags "[-][?]\(\s\+\|$\)"
syn match povrUnixFlags "[-]\{1,2\}libraries\(\s\+\|$\)"
syn match povrUnixFlags "[-]\{1,2\}license\(\s\+\|$\)"
" Linux/Unix --version and -version collide with ini keyword so not highlighted
" as a povr Unix flag.

" POV-Ray (povr form) Unix / linux command line flags which are errors
" ===================================================

" The --benchmark|-benchmark flag. The povr fork removed this feature.
syn match povrUnixFlagErrors "[-]\{1,2\}benchmark\(\s\+\|$\)"

" The -generation flavor of flag not supported?. Only '-' form not.
syn match povrUnixFlagErrors "\(^\|\s+\)[-]\{1\}generation\(\s\+\|$\)"

" Command line / ini file keywords also POV-Ray v3.8 / povr v3.8 keywords 
" ====================================
syn keyword povriniKeywordArgs false no off on true yes bt2020 bt709 srgb
" Links to highlights. Mostly not aligning to predefined highlight categories.
" With POV-Ray, povr included, keywords often fit multiple categories. Herein 
" only aiming for generalized highlighting buckets. 
hi def link povriniComment          Comment
hi def link povriniTodo             Todo
hi def link povriniKeywords         Type
hi def link povriniFlags            Identifier
hi def link povriniKeywordArgs      PreProc
hi def link povriniString           Constant
hi def link povrUnixFlags           Statement

hi def link povriniCommentErrors    Error
hi def link povriniKeywordErrors    Error
hi def link povriniFlagErrors       Error
hi def link povrUnixFlagErrors      Error
hi def link povriniStringErrors     Error

" For smallish ini files expect performance of fromstart OK.
syntax sync fromstart

let b:current_syntax = "povrini"

let &cpo = s:cpo_save
unlet s:cpo_save

" SPDX-License-Identifier: CC-BY-SA-4.0
